//**********************************************************************
// MapNetwork
// ==========
//
// John Rennie
// jrennie@cix.compulink.co.k
// 27th March 1999
//**********************************************************************

#ifndef STRICT
#define STRICT
#endif

#include <windows.h>
#include <stdio.h>
#include <stdarg.h>


//**********************************************************************
// Function prototypes
// -------------------
//**********************************************************************

BOOL ProcessGroup(const char* Group, const char* ControlFile);
DWORD SubstituteEnvironmentVariables(char* InBuf, char* OutBuf, DWORD MaxLen);
const char* GetLastErrorMessage(void);


//**********************************************************************
// Messages
// --------
//**********************************************************************

#define String_IDS_ERRDISCONNECT "Error disconnecting %s %s\n"
#define String_IDS_ERRCONNECT "Error connecting %s to %s: %s\n"


//**********************************************************************
// main
// ----
//**********************************************************************

int main(int argc, char* argv[])
{ int i, j;
  DWORD d;
  char username[128], group[128], computername[128], s[1024];
  char path[MAX_PATH+1], groups[MAX_PATH+1], maps[MAX_PATH+1];

// Get the group info and resource map files

  GetModuleFileName(NULL, path, MAX_PATH);
  for (i = 0; path[i] != '\0'; i++);
  while (i > 0 && path[i-1] != '\\') i--;
  path[i] = '\0';

  lstrcpy(groups, path);
  lstrcat(groups, "MapNetworkGroup.ini");

  lstrcpy(maps, path);
  lstrcat(maps, "MapNetworkResource.ini");

// Do the "Everyone" mappings

  ProcessGroup("Everyone", maps);

// Do the computer name mappings

  lstrcpy(computername, "COMPUTER:");
  d = 127 - lstrlen(computername);

  if (GetComputerName(computername + lstrlen(computername), &d))
    ProcessGroup(computername, maps);

// Get the username and use it to work through the groups

  if (GetEnvironmentVariable("USERNAME", username, 127) > 0)
  { if (GetPrivateProfileString("GroupInfo", username, "", s, 1023, groups) > 0)
    { i = 0;

// Keep going till we hit a null string which marks the end

      while (s[i] != '\0')
      { lstrcpy(group, "GROUP:");
        for (j = lstrlen(group); s[i] != ',' && s[i] != '\0'; i++, j++)
          group[j] = s[i];
        group[j] = '\0';

        if (s[i] == ',')
          s[i++] = '\0';

        ProcessGroup(group, maps);
      }
    }

// Finally process the username

  lstrcpy(username, "USER:");

  if (GetEnvironmentVariable("USERNAME", username + lstrlen(username), 127 - lstrlen(username)) > 0)
    ProcessGroup(username, maps);
  }

// All done

  return 0;
}


//**********************************************************************
// ProcessGroup
// ------------
//**********************************************************************

BOOL ProcessGroup(const char* Group, const char* ControlFile)
{ DWORD d;
  NETRESOURCE nr;
  char* p;
  char unc[MAX_PATH+1], bare_unc[MAX_PATH+1], keys[1024];

// Get the list of keys for this group

  GetPrivateProfileString(Group, NULL, "", keys, 1023, ControlFile);

  for (p = keys; *p != '\0'; p += lstrlen(p) + 1)
  {

// A key with no value means disconnect the resource

    if (GetPrivateProfileString(Group, p, "", bare_unc, MAX_PATH, ControlFile) == 0)
    { d = WNetCancelConnection2(p, CONNECT_UPDATE_PROFILE, TRUE);
      if (d != NO_ERROR)
        printf(String_IDS_ERRDISCONNECT, p, GetLastErrorMessage());
    }

// Connect the resource; if it's already connected then reconnect it

    else
    { SubstituteEnvironmentVariables(bare_unc, unc, MAX_PATH);

      FillMemory(&nr, sizeof(nr), 0);
      nr.dwType = p[1] == ':' ? RESOURCETYPE_DISK : RESOURCETYPE_PRINT;
      nr.lpLocalName = p;
      nr.lpRemoteName = unc;

      d = WNetAddConnection2(&nr, NULL, NULL, 0);
      if (d == ERROR_ALREADY_ASSIGNED)
      { WNetCancelConnection2(p, CONNECT_UPDATE_PROFILE, TRUE);
        d = WNetAddConnection2(&nr, NULL, NULL, 0);
      }

      if (d != NO_ERROR)
        printf(String_IDS_ERRCONNECT, p, unc, GetLastErrorMessage());
    }
  }

// All done

  return TRUE;
}


//**********************************************************************
// SubstituteEnvironmentVariables
// ------------------------------
//**********************************************************************

DWORD SubstituteEnvironmentVariables(char* InBuf, char* OutBuf, DWORD MaxLen)
{ char env[256], def[256], envval[256];
  char *p_in, *p_out, *p_env, *p_def;
  DWORD maxlen, len, d;

/*** Initialise pointers to start of input and output buffers */

  p_out  = OutBuf;
  p_in   = InBuf;
  maxlen = MaxLen - 1;
  len    = 0;

/*** Loop copying data from inbuf to outbuf */

  while (*p_in != '\0')
  {

/*** If the character is not a % then just copy it */

    if (*p_in != '%')
    { if (len >= maxlen) // Remember to check the length
      { *p_out = '\0';
        return(0);
      }
      else
      { *p_out++ = *p_in++;
        len++;
      }
    }

/*** If the next two characters are %% then copy a single % */

    else if (*(p_in+1) == '%')
    { if (len >= maxlen) // Remember to check the length
      { *p_out = '\0';
        return(0);
      }
      else
      { *p_out++ = *p_in++;
        *p_in++;
        len++;
      }
    }

/*** We have %something so look for an environment variable */

    else
    { p_in++;

/*** Copy everything up to the next % into the env var buffer */

      for (p_env = env; *p_in != '%' && *p_in != '\0'; *p_env++, *p_in++)
        *p_env = *p_in;
      *p_env = '\0';

/*** If a second % was not found just copy the characters to outbuf */

      if (*p_in != '%')
      { if (len >= maxlen) // Remember to check the length
        { *p_out = '\0';
          return(0);
        }
        *p_out++ = '%';
        len++;

        for (p_env = env; *p_env != '\0'; *p_env++, *p_out++)
        { if (len >= maxlen) // Remember to check the length
          { *p_out = '\0';
            return(0);
          }
          else
          { *p_out = *p_env;
          }
        }
      }

/*** OK, we have a substring %xxx% with xxx now in env */

      else
      {

/*** Look for a default character '|' */

        p_def = def;
        *p_def = '\0';

        for (p_env = env; *p_env != '|' && *p_env != '\0'; p_env++);

/*** If there is a '|' then split into env var name and default value */

        if (*p_env == '|')
        { *p_env++ = '\0';
          while (*p_env != '\0')
            *p_def++ = *p_env++;
          *p_def = '\0';
        }

/*** Right, get the env var value */

        d = GetEnvironmentVariable(env, envval, 256);

/*** If the env var was found then copy into outbuf */

        if (d > 0)
        { for (p_env = envval; *p_env!= '\0'; p_env++, p_out++, len++)
          { if (len >= maxlen) // Remember to check the length
            { *p_out = '\0';
              return(0);
            }
            else
            { *p_out = *p_env;
            }
          }
        }

/*** If not found use the default (if any) */

        else if (def[0] != '\0')
        { for (p_def = def; *p_def != '\0'; p_def++, p_out++, len++)
          { if (len >= maxlen) // Remember to check the length
            { *p_out = '\0';
              return(0);
            }
            else
            { *p_out = *p_def;
            }
          }
        }

/*** Finally remember to step past the trailing % */

		p_in++;
      }
    }
  }

/*** Terminate the string */

  *p_out = '\0';

/*** Return the length of the completed string */

  return(len);
}


//**********************************************************************
// GetLastErrorMessage
// -------------------
//**********************************************************************

const char* GetLastErrorMessage(void)
{ DWORD d;
  static char errmsg[1024];

  d = GetLastError();

  strcpy(errmsg, "<unknown error>");
  FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 0, d, 0, errmsg, 1023, NULL);
  return(errmsg);
}


