//**********************************************************************
// MapNetworkUpdate
// ================
//
// John Rennie
// jrennie@cix.compulink.co.k
// 27th March 1999
//**********************************************************************

#ifndef STRICT
#define STRICT
#endif

#include <windows.h>
#include <lmaccess.h>
#include <lmerr.h>
#include <stdio.h>
#include <stdarg.h>


//**********************************************************************
// Function prototypes
// -------------------
//**********************************************************************

BOOL CreateLoginScript(const char* Username);
const char* GetLastErrorMessage(void);


//**********************************************************************
// main
// ----
//**********************************************************************

int main(int argc, char* argv[])
{ int i;
  NET_API_STATUS nas;
  WCHAR* domain;
  char username[32], group[32];
  char path[MAX_PATH+1], groupfile[MAX_PATH+1], mapfile[MAX_PATH+1], s[256];

  DWORD entriesread, totalentries, resume_handle, usernum;
  USER_INFO_0* p_usernames;

  DWORD groupsread, totalgroups, groupnum;
  GROUP_USERS_INFO_0* p_groups;

// Get the group info and resource map files

  GetModuleFileName(NULL, path, MAX_PATH);
  for (i = 0; path[i] != '\0'; i++);
  while (i > 0 && path[i-1] != '\\') i--;
  path[i] = '\0';

  lstrcpy(groupfile, path);
  lstrcat(groupfile, "MapNetworkGroup.ini");

  lstrcpy(mapfile, path);
  lstrcat(mapfile, "MapNetworkResource.ini");

// Find the domain controller

  nas = NetGetDCName(NULL, NULL, (LPBYTE*) &domain);
  if (nas != NERR_Success)
  { printf("Cannot get DC name: %s\n", GetLastErrorMessage());
    return 2;
  }

  WideCharToMultiByte(CP_ACP, 0, (WCHAR*) domain, -1, s, 255, NULL, NULL);
  printf("Domain Controller = %s\n", s);

// Back up then delete the old group file

  lstrcpy(s, groupfile);
  lstrcat(s, ".old");
  CopyFile(groupfile, s, FALSE);
  DeleteFile(groupfile);

// Enumerate all the users

  resume_handle = 0;

  do
  { nas = NetUserEnum(domain, 0, FILTER_NORMAL_ACCOUNT, (LPBYTE*) &p_usernames, 0x10000, &entriesread, &totalentries, &resume_handle);
    if (nas != NERR_Success && nas != ERROR_MORE_DATA)
    { printf("NetUserEnum call failed: %s\n", GetLastErrorMessage());
      return 2;
    }

    for (usernum = 0; usernum < entriesread; usernum++)
    { WideCharToMultiByte(CP_ACP, 0, (WCHAR*) p_usernames[usernum].usri0_name, -1, username, 31, NULL, NULL);

// First create the login script for the user

      CreateDirectory("newscripts", NULL);
      CreateLoginScript(username);

// Now add the user's groups to the group data file

      nas = NetUserGetGroups(domain, p_usernames[usernum].usri0_name, 0, (LPBYTE*) &p_groups, 0x10000, &groupsread, &totalgroups);

      if (nas != NERR_Success && nas != ERROR_MORE_DATA)
      { printf("NetUserEnum call for user %s failed: %s\n", username, GetLastErrorMessage());
      }
      else
      { lstrcpy(s, "");

        for (groupnum = 0; groupnum < groupsread; groupnum++)
        { WideCharToMultiByte(CP_ACP, 0, (WCHAR*) p_groups[groupnum].grui0_name, -1, group, 31, NULL, NULL);

          if (lstrlen(s) > 0)
            lstrcat(s, ",");
          lstrcat(s, group);
        }

        WritePrivateProfileString("GroupInfo", username, s, groupfile);
      }
    }
  } while (nas == ERROR_MORE_DATA);

// All done

  return 0;
}


//**********************************************************************
// CreateLoginScript
// -----------------
//**********************************************************************

BOOL CreateLoginScript(const char* Username)
{ FILE* f;
  char s[MAX_PATH+1];

  lstrcpy(s, "newscripts\\");
  lstrcat(s, Username);
  lstrcat(s, ".bat");

  f = fopen(s, "w");
  if (f)
  { fprintf(f, "if \"%%OS%%\" == \"Windows_NT\" goto NT\n");
    fprintf(f, "set USERNAME=%s\n\n", Username);
    fprintf(f, ":NT\n");
    fprintf(f, "%%0\\..\\MapNetworkLogin.bat\n");
  }

  return TRUE;
}


//**********************************************************************
// GetLastErrorMessage
// -------------------
//**********************************************************************

const char* GetLastErrorMessage(void)
{ DWORD d;
  static char errmsg[1024];

  d = GetLastError();

  strcpy(errmsg, "<unknown error>");
  FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 0, d, 0, errmsg, 1023, NULL);
  return(errmsg);
}


